*:****************************************************************************
*: Procedure file: DLOCK.PRG
*:
*:         System: FoxPRO DBF/IDX File Lock Status
*:         Author: Bruce K. Taub       (CIS Id#: 70742,443)
*:      Copyright: (c) 1992, AIM Computer Associates, Inc.
*:  Last modified: 02/01/92     17:48
*:
*:  Procs & Fncts: STRIPTAG()
*:               : STRIPDIR()
*:               : DLOCK_LO()
*:
*:          Calls: STRIPTAG()         (function  in DLOCK.PRG)
*:               : STRIPDIR()         (function  in DLOCK.PRG)
*:               : DLOCK_LO()         (function  in DLOCK.PRG)
*:
*: This function returns a result code corresponding to a file's usage status
*: on a network.  This is very helpful in determining if another user has
*: SHARED or EXCLUSIVE use of a file without relying on an ON ERROR routine.
*: 
*:****************************************************************************
*  Function dLock(dbfname)
*
*  return codes:     0 - File Not In Use
*                    1 - Local Exclusive File USE/d
*                    2 - Local Shared USE
*                    3 - Network Exclusive File Lock
*                    4 - Network Shared USE
*                    5 - File Not Found
*                    6 - Local & Network Shared Use
*                    9 - No Parameter Passed
******************************************************************************
* Variable                     Usage
* ============================================================================
* dlock_ar                   - current SELECT (workarea)
* dlock_cr                   - current Record (if eof(), max record number)
* dlock_d2                   - counter
* dlock_dc                   - workarea counter
* dlock_df                   - file name (with extension)
* dlock_dz                   - idx flag
* dlock_fl                   - current filter
* dlock_in                   - file name to check (extension is optional)
* dlock_nd                   - current INDEX file
* dlock_or                   - current CDX/IDX Tag
* dlock_rt                   - the return value (see list above for meaning)
******************************************************************************
parameters dlock_in                               && DBF Name (String)
if parameters()=0                                 && No Parameter/dbf name passed
  dlock_rt=9
else
  if striptag(dlock_in)=dlock_in                  && File Name Have ".DBF" Extension?
    dlock_df=lower(dlock_in+iif(right(dlock_in,1)=".","",".")+"dbf")
    dlock_in=stripdir(lower(dlock_in))
  else
    dlock_df=lower(dlock_in)
    dlock_in=stripdir(striptag(lower(dlock_in)))
  endif (striptag(dlock_in)=dlock_in           && File Name Have ".DBF" Extension?)
  do case
  case ! file(dlock_df)                        && does file exist
    dlock_rt=5
  case at(".dbf",dlock_df)>0 .and. used(dlock_in) && DBF / Alias Check
    dlock_ar=select()                            && save current workarea
    select (dlock_in)
    dlock_rt=iif(sys(2011)="Exclusive",1,2)
    *************************************
    * If file is not used Exclusively
    * then check to see if there is
    * any other users accessing the file
    *************************************
    if dlock_rt=2                                && test for multiple Users
      dlock_nd=stripdir(striptag(ndx(1)))        && index name
      dlock_or=order()                           && tag (if no IDX)
      dlock_cr=min(recno(),reccount())           && current record (no phantoms)
      dlock_fl=filter()                          && current filter
      use                                        && release the dbf
      dlock_rt=iif(dlock_lo(dlock_df)=0,2,6)     && check status again
      do case
      case !empty(dlock_nd)                      && reset IDX (if available)
        use (dlock_df) index (dlock_or)
      case !empty(dlock_or)                      && reset ORDER (if available)
        use (dlock_df) order (dlock_or)
      otherwise                                  && use the database
        use (dlock_df)
      endcase
      goto dlock_cr                              && reset position in dbf
      if !empty(dlock_fl)                        && reset filter
        set filter to &dlock_fl
      endif (!empty(dlock_fl))
    endif (dlock_rt=2              && test for both)
    select int(dlock_ar)                         && reset workarea
  case at(".idx",dlock_df)>0                     && IDX Check
    dlock_dz=.f.                                 && test for local IDX usage
    for dlock_dc=1 to 25
      dlock_d2=1
      do while !empty(ndx(dlock_d2,dlock_dc))
        if stripdir(lower(ndx(dlock_d2,dlock_dc)))=dlock_df
          dlock_dz=.t.
          exit
        endif (stripdir(lower(ndx(dlock_d2,dlock_dc)))=dlock_df)
        dlock_d2=dlock_d2+1
      enddo
      if dlock_dz
        dlock_rt=1
        exit
      endif (dlock_dz)
    endfor (dlock_dc)
    if type("dlock_rt")#"N"                     && test for network IDX usage
      dlock_rt=dlock_lo(dlock_df)
    endif (type("dlock_rt")#"N")
  otherwise
    dlock_rt=dlock_lo(dlock_df)                 && test for file usage
  endcase
endif (parameters()=0                          && No Parameter/dbf name passed)
return dlock_rt
*!*****************************************************************************
*!
*!       Function: DLOCK_LO()
*!
*!      Called by: DLOCK.PRG                          
*!
*!    Other Files: DLOCK_L1
*!
*!*****************************************************************************
******************************************************************************
* Variable                     Usage
* ============================================================================
* dlock_fh                    - Low-Level File Handle
* dlock_fr                    - Low-Level File Close Return Value
* dlock_l1                    - Full File Name
* dlock_rz                    - Return Value
******************************************************************************
function dlock_lo
  parameters dlock_l1
  dlock_fh=fopen(dlock_l1,0)                     && Network Exclusive Use
  if dlock_fh <0 .and. (ferror()=5 .or. ferror()=31)
    dlock_rz=3
  else
    dlock_fr=fclose(dlock_fh)
    dlock_fh=fopen(dlock_l1,2)                   && Network Shared use
    if dlock_fh <0 .and. (ferror()=5 .or. ferror()=31)
      dlock_rz=4
    else
      dlock_fr=fclose(dlock_fh)                  && no lock
      dlock_rz=0
    endif (dlock_fh <0 .and. (ferror()=5 .or. ferror()=31))
  endif (dlock_fh <0 .and. (ferror()=5 .or. ferror()=31))
return dlock_rz

*!*****************************************************************************
*! Remove Directory Prefix from File Name
*!
*!       Function: STRIPDIR()
*!
*!      Called by: DLOCK.PRG                          
*!
*!*****************************************************************************
******************************************************************************
* Variable                     Usage
* ============================================================================
* tripdir_ot                  - Return Value
* tripdir_sd                  - File Name
******************************************************************************
function stripdir
  parameters tripdir_sd
  do case
  case "\" $ tripdir_sd               && filename starts after last backslash
    tripdir_ot=substr(tripdir_sd,rat("\",tripdir_sd)+1)
  case ":" $ tripdir_sd               && filename starts after colen
    tripdir_ot=substr(tripdir_sd,at(":",tripdir_sd)+1)
  otherwise
    tripdir_ot=tripdir_sd             && filename does not have directory/path
  endcase
return tripdir_ot
*!*****************************************************************************
*! Remove File Extension from File Name
*!
*!       Function: STRIPTAG()
*!
*!      Called by: DLOCK.PRG                          
*!
*!*****************************************************************************
******************************************************************************
* Variable                     Usage
* ============================================================================
* triptag_ot                  - Return Value
* triptag_sd                  - File Name
******************************************************************************
function striptag
  parameters triptag_st
  if "." $ triptag_st                 && extension starts after last period
    triptag_ot=substr(triptag_st,1,rat(".",triptag_st)-1)
  else
    triptag_ot=triptag_st
  endif ("." $ triptag_st)
return triptag_ot
*:
*: EOF: DLOCK.PRG
